
#import <appkit/Application.h>
#import <appkit/nextstd.h>
#import <objc/Storage.h>
#import "PlotterXModule.h"
#import "PlotterXModuleConstants.h"
#import "PlotView.h"

/*
 * Define the list of commands and functions this XModule implements
 */

/*
 * Keep track of the mach header where we get loaded
 * so we can use it to load other stuff later (such as
 * nib files)
 * Note: only needed in XModules that load nib files, etc.
 */

	const char *handlerList[] = {
		"clearplotwindow",
		"closeplotwindow",
		"plot",
		"setplotwindowtitle",
		"PLOTPOINTS",
		"PV_VERSION",
		NULL
	};

static struct mach_header	*loadHeader = NULL;

@implementation PlotterXModule : XModule

- init
{
	[super init];
	
	// Do your stuff here...
	
	return ( self );
}

- free
{
	// Do your stuff here...

	return ( [super free] );
}

+ (const char *)moduleName
{
	return ( "PlotterXModule" );
}

+ (const char **)xCmdsAndFcns
{
	return ( handlerList );
}

- prepareToExecuteHandlers
{
	// [self loadInterfaceData];
	return ( self );
}

- executeHandler:(NXAtom)handlerName
{
	if (!strcmp(handlerName, "clearplotwindow")) {
		return ( [self clearPlotWindow] );
	} else if (!strcmp(handlerName, "closeplotwindow")) {
		return ( [self closePlotWindow] );
	} else if (!strcmp(handlerName, "plot")) {
		return ( [self plot] );
	} else if (!strcmp(handlerName, "setplotwindowtitle")) {
		return ( [self setPlotWindowTitle] );
	} else if (!strcmp(handlerName, "PLOTPOINTS")) {
		return ( [self plotPoints] );
	} else if (!strcmp(handlerName, "PV_VERSION")) {
		return ( [self pv_version] );
	} else {
		return ( nil ); // handlerName not recognized!
	}
}

+ finishLoading:(struct mach_header *)header
{
	loadHeader = header; // Save this for later use
	return ( self );
}

- loadInterfaceData
{
	//[NXApp loadNibSection:"MyXModule.nib" owner:self
	//    withNames:NO fromHeader:loadHeader] )
		
	return ( self );
}

// XModule specific code...
//
- plot
{
	int			pCount = [self paramCount];
	NXRect		winFrame;
	int			lineCounter = 1;
	char		*currLine;
	NXPoint		currPt;

	if ( pCount != 1 ) {
		return ([[self getEmptyContainer] setString:
				"Usage:  say <aStringToConvertToSpeech>"]);
	}

	// If we don't already have a window/plotview, then
	//   go about creating it, and making the plotview the
	//   content view of the window.  Wacked, but works.
	if ( !myNewPlotWindow ) {
		// All is okay, so now create a window
		NXSetRect(&winFrame, 30.0, 30.0, 400.0, 400.0);
		myNewPlotWindow = [[Window alloc] initContent:&winFrame
				style:NX_RESIZEBARSTYLE
				backing:NX_BUFFERED
				buttonMask:(NX_CLOSEBUTTONMASK|NX_MINIATURIZEBUTTONMASK)
				defer:YES];

		// ... add a plotView to it ...
		myNewPlotView = [[PlotView alloc] init];
		if ( !myNewPlotView ) {
			NXLogError("Error: could not init a new plotView.\n");
			return ( self );
		}
		[myNewPlotWindow setDelegate:self];
		[myNewPlotWindow setTitle:"Plot Window"];
		[myNewPlotWindow setContentView:myNewPlotView];
		[myNewPlotWindow center];
	}

	// Make all the existing (if any) points go away.
	[myNewPlotView clear:self];
	[myNewPlotView display];

	// ... scan and plot send the points to the plotView ...
	// NOTE: Can loop forever...  Make sure it exits...
	for ( lineCounter = 1 ; ; lineCounter++ ) {
		// Use XMod call back to get the line chunk...
		currLine = [[self getParam:1]
				copyChunkFromChar:CHUNK_NONE to:CHUNK_NONE
				ofWord:CHUNK_NONE to:CHUNK_NONE
				ofItem:CHUNK_NONE to:CHUNK_NONE
				ofLine:lineCounter to:lineCounter];
		if (strlen(currLine) == 0) break;
		sscanf(currLine, " %f , %f ", &currPt.x, &currPt.y);
		[myNewPlotView registerPoint:&currPt];
		free(currLine);	
	}

	[[myNewPlotWindow orderFront:self] displayIfNeeded];
	[myNewPlotView display];

	// ... and return a status.
	return ( [[self getEmptyContainer] setString:
			"<returnValue>"] );
}

- clearPlotWindow
{
	if ( myNewPlotView ) [myNewPlotView clear:self];

	return ( [[self getEmptyContainer] setString: ""] );
}

- setPlotWindowTitle
{
	int		pCount = [self paramCount];
	char	oldTitle[1024];

	if (pCount != 1) {
		return ( [[self getEmptyContainer] setString:
				"Usage:  setPlotWindowTitle <windowTitle>"] );
	}

	if ( myNewPlotWindow ) {
		strcpy(oldTitle, [myNewPlotWindow title]);
		[myNewPlotWindow setTitle:[[self getParam:1] stringVal]];
	}

	return ( [[self getEmptyContainer] setString: oldTitle] );
}

- closePlotWindow
{
	if ( myNewPlotWindow )
		[myNewPlotWindow performClose:self];

	return ( self );
}

- plotPoints
{
	id		returnValue = [[self getEmptyContainer] setString:""];
	char	buffer[256];
	int		i;
	id		plotPoints;
	NXPoint	*aPoint;

	if ( !myNewPlotView )
		return ( [[self getEmptyContainer] setString:""] );

	plotPoints = [myNewPlotView points];
	i = [plotPoints count];
	while (i--) {
		aPoint = (NXPoint *)[plotPoints elementAt:i];
		sprintf(buffer, "%f,%f\n", aPoint->x, aPoint->y);
		[returnValue appendString:buffer];
	}

	return ( returnValue );

}

- pv_version
{
	id		version;

	version = [[ self getEmptyContainer ] setString:"r" ];
	[ version appendString:PV_RELEASE ];
	[ version appendString:" v" ];
	[ version appendString:PV_VERSION ];
	[ version appendString:"    " ];
	[ version appendString:PV_RELTIME ];
	[ version appendString:"\n" ];
	[ version appendString:PV_COPYRIGHT ];

	return ( version );
}


// Delegate Methods
//
- windowWillClose:sender
{
	if ( [[sender contentView] class] == [PlotView class] ) {
		myNewPlotWindow = nil;
		myNewPlotView = nil;
	}

	return ( self );
}

@end

